import customtkinter as ctk
from tkinter import messagebox, filedialog
import requests
import zipfile
import os
import threading
import time
from pathlib import Path
import sys
from PIL import Image, ImageTk

# Set appearance mode and color theme
ctk.set_appearance_mode("dark")
ctk.set_default_color_theme("blue")

def resource_path(relative_path):
    """Get absolute path to resource, works for dev and for PyInstaller"""
    try:
        # PyInstaller creates a temp folder and stores path in _MEIPASS
        base_path = sys._MEIPASS
    except Exception:
        base_path = os.path.abspath(".")
    
    return os.path.join(base_path, relative_path)

class CompactPredictDownloader:
    def __init__(self):
        self.setup_window()
        self.setup_variables()
        self.load_inter_font()
        self.create_widgets()
        
        # Default destination
        self.destination_entry.insert(0, "xpand-extension")
        
    def setup_window(self):
        self.app = ctk.CTk()
        self.app.title("Predict Assistant - Extension Downloader")
        self.app.geometry("850x650")  # Slightly larger for better readability
        
        try:
            # Untuk Windows/Linux
            icon_path = resource_path("favicon.ico")
            if os.path.exists(icon_path):
                self.app.iconbitmap(icon_path)
            
            # Untuk semua platform (termasuk macOS)
            icon_image_path = resource_path("logo-predict-p.png")
            if os.path.exists(icon_image_path):
                icon_image = Image.open(icon_image_path)
                photo_icon = ImageTk.PhotoImage(icon_image)
                self.app.iconphoto(True, photo_icon)
        except Exception as e:
            print(f"❌ Gagal memuat ikon: {e}")

        # Set window icon and make it resizable
        self.app.resizable(True, True)
        
        # Configure grid with proper weights
        self.app.grid_columnconfigure(0, weight=1)
        self.app.grid_rowconfigure(0, weight=1)
        
    def load_inter_font(self):
        """Load Inter font from OTF files"""
        try:
            # Try to load Inter fonts
            font_files = [
                "Inter-Regular.otf",
                "Inter-Medium.otf", 
                "Inter-SemiBold.otf",
                "Inter-Bold.otf"
            ]
            
            # Check if font files exist using resource_path
            self.inter_available = all(os.path.exists(resource_path(font)) for font in font_files)
            
            if self.inter_available:
                self.font_family = "Inter"
                print("✅ Inter font loaded successfully")
            else:
                # Fallback fonts
                if sys.platform.startswith('win'):
                    self.font_family = "Segoe UI"
                elif sys.platform.startswith('darwin'):
                    self.font_family = "SF Pro Display"
                else:
                    self.font_family = "Ubuntu"
                print(f"⚠️ Using fallback font: {self.font_family}")
                    
        except Exception as e:
            print(f"❌ Font loading error: {e}")
            self.font_family = "default"
    
    def setup_variables(self):
        self.source_url = "http://notmastah.dev/extensions/predict-assistant/extension.zip"
        self.status_var = ctk.StringVar(value="Ready to download Predict Assistant extension")
        self.step_var = ctk.StringVar(value="Ready • Click download to begin")
        self.is_downloading = False
        
    def create_widgets(self):
        # Main container
        main_frame = ctk.CTkFrame(
            self.app,
            fg_color="#000000",
            corner_radius=0
        )
        main_frame.grid(row=0, column=0, sticky="nsew")
        main_frame.grid_columnconfigure(0, weight=1)
        main_frame.grid_rowconfigure(2, weight=1)  # Make logs expand
        
        # Compact header
        self.create_compact_header(main_frame)
        
        # Main control panel
        self.create_control_panel(main_frame)
        
        # Compact logs
        self.create_compact_logs(main_frame)
        
        # Minimal footer
        self.create_minimal_footer(main_frame)
    
    def create_compact_header(self, parent):
        # Compact header section
        header_frame = ctk.CTkFrame(
            parent,
            fg_color="#0a0a0a",
            # corner_radius=16,
            border_width=1,
            border_color="#1a1a1a",
            height=100  # Slightly taller for better spacing
        )
        header_frame.grid(row=0, column=0, sticky="ew", pady=(0, 16))
        header_frame.grid_columnconfigure(0, weight=1)
        header_frame.grid_propagate(False)
        
        # Content container
        content_frame = ctk.CTkFrame(
            header_frame,
            fg_color="transparent"
        )
        content_frame.grid(row=0, column=0, sticky="ew", padx=24, pady=20)  # More padding
        content_frame.grid_columnconfigure(0, weight=1)
        
        # Company badge and title in one row
        title_row = ctk.CTkFrame(
            content_frame,
            fg_color="transparent"
        )
        title_row.grid(row=0, column=0, sticky="ew")
        title_row.grid_columnconfigure(1, weight=1)
        
        # Compact badge
        try:
            # Muat ikon
            icon_path = resource_path("logo-predict-p.png")
            if os.path.exists(icon_path):
                icon_image = Image.open(icon_path)
                icon_image = icon_image.resize((28, 28), Image.LANCZOS)  # Bigger icon
                icon_photo = ImageTk.PhotoImage(icon_image)
                
                badge_label = ctk.CTkLabel(
                    title_row,
                    image=icon_photo,
                    text="Predict",
                    compound="left",
                    font=ctk.CTkFont(family=self.font_family, size=14, weight="bold"),  # Larger font
                    text_color="#0070f3",
                    fg_color="#111111",
                    corner_radius=8,
                    width=100,
                    height=32
                )
                # Simpan referensi gambar untuk mencegah garbage collection
                badge_label.icon_image = icon_photo
            else:
                # Fallback jika file ikon tidak ditemukan
                badge_label = ctk.CTkLabel(
                    title_row,
                    text="⚡ Predict",
                    font=ctk.CTkFont(family=self.font_family, size=14, weight="bold"),
                    text_color="#0070f3",
                    fg_color="#111111",
                    corner_radius=8,
                    width=80,
                    height=32
                )
        except Exception as e:
            print(f"❌ Gagal memuat ikon badge: {e}")
            badge_label = ctk.CTkLabel(
                title_row,
                text="⚡ Predict",
                font=ctk.CTkFont(family=self.font_family, size=14, weight="bold"),
                text_color="#0070f3",
                fg_color="#111111",
                corner_radius=8,
                width=80,
                height=32
            )
        
        badge_label.grid(row=0, column=0, sticky="w", padx=(0, 16))
        
        # Main title
        title_label = ctk.CTkLabel(
            title_row,
            text="Extension Downloader",
            font=ctk.CTkFont(family=self.font_family, size=28, weight="bold"),  # Larger title
            text_color="#ffffff"
        )
        title_label.grid(row=0, column=1, sticky="w")
        
        # Compact description
        desc_label = ctk.CTkLabel(
            content_frame,
            text="Download and extract Predict Assistant extension package",
            font=ctk.CTkFont(family=self.font_family, size=14),  # Larger description
            text_color="#888888"
        )
        desc_label.grid(row=1, column=0, sticky="w", pady=(10, 0))
    
    def create_control_panel(self, parent):
        # Compact control panel
        control_frame = ctk.CTkFrame(
            parent,
            fg_color="#0a0a0a",
            # corner_radius=16,
            border_width=1,
            border_color="#1a1a1a"
        )
        control_frame.grid(row=1, column=0, sticky="ew", pady=(0, 16))
        control_frame.grid_columnconfigure(0, weight=1)
        
        # Content with more padding for readability
        content_frame = ctk.CTkFrame(
            control_frame,
            fg_color="transparent"
        )
        content_frame.grid(row=0, column=0, sticky="ew", padx=24, pady=20)
        content_frame.grid_columnconfigure(0, weight=1)
        
        # Source info in compact format
        source_frame = ctk.CTkFrame(
            content_frame,
            fg_color="#111111",
            corner_radius=12,
            height=80  # Taller for better readability
        )
        source_frame.grid(row=0, column=0, sticky="ew", pady=(0, 16))
        source_frame.grid_columnconfigure(1, weight=1)
        source_frame.grid_propagate(False)
        
        source_icon = ctk.CTkLabel(
            source_frame,
            text="📦",
            font=ctk.CTkFont(family=self.font_family, size=20),  # Larger icon
            text_color="#0070f3",
            width=50
        )
        source_icon.grid(row=0, column=0, padx=(20, 12), pady=16)
        
        source_info = ctk.CTkFrame(
            source_frame,
            fg_color="transparent"
        )
        source_info.grid(row=0, column=1, sticky="ew", pady=12, padx=(0, 20))
        
        source_label = ctk.CTkLabel(
            source_info,
            text="Package Source",
            font=ctk.CTkFont(family=self.font_family, size=14, weight="bold"),  # Larger font
            text_color="#ffffff"
        )
        source_label.pack(anchor="w")
        
        source_url = ctk.CTkLabel(
            source_info,
            text=self.source_url,
            font=ctk.CTkFont(family="Consolas" if sys.platform.startswith('win') else "Monaco", size=12),  # Larger monospace
            text_color="#888888"
        )
        source_url.pack(anchor="w", pady=(4, 0))
        
        # Destination section
        dest_label = ctk.CTkLabel(
            content_frame,
            text="📁 Installation Directory",
            font=ctk.CTkFont(family=self.font_family, size=15, weight="bold"),  # Larger font
            text_color="#ffffff"
        )
        dest_label.grid(row=1, column=0, sticky="w", pady=(0, 10))
        
        # Destination input row
        dest_row = ctk.CTkFrame(
            content_frame,
            fg_color="transparent"
        )
        dest_row.grid(row=2, column=0, sticky="ew", pady=(0, 20))
        dest_row.grid_columnconfigure(0, weight=1)
        
        self.destination_entry = ctk.CTkEntry(
            dest_row,
            placeholder_text="Enter installation directory...",
            font=ctk.CTkFont(family=self.font_family, size=13),  # Larger input font
            height=44,  # Taller input
            corner_radius=12,
            fg_color="#111111",
            border_color="#333333",
            text_color="#ffffff"
        )
        self.destination_entry.grid(row=0, column=0, sticky="ew", padx=(0, 12))
        
        browse_btn = ctk.CTkButton(
            dest_row,
            text="📂",
            font=ctk.CTkFont(family=self.font_family, size=16),
            width=44,
            height=44,
            corner_radius=12,
            fg_color="#1a1a1a",
            hover_color="#2a2a2a",
            command=self.browse_destination
        )
        browse_btn.grid(row=0, column=1)
        
        # Progress section
        progress_frame = ctk.CTkFrame(
            content_frame,
            fg_color="transparent"
        )
        progress_frame.grid(row=3, column=0, sticky="ew", pady=(0, 20))
        progress_frame.grid_columnconfigure(0, weight=1)
        
        # Status and progress in compact layout
        status_row = ctk.CTkFrame(
            progress_frame,
            fg_color="transparent"
        )
        status_row.grid(row=0, column=0, sticky="ew", pady=(0, 10))
        status_row.grid_columnconfigure(0, weight=1)
        
        self.step_label = ctk.CTkLabel(
            status_row,
            textvariable=self.step_var,
            font=ctk.CTkFont(family=self.font_family, size=14, weight="bold"),  # Larger status font
            text_color="#ffffff"
        )
        self.step_label.grid(row=0, column=0, sticky="w")
        
        # Progress bar
        self.progress_bar = ctk.CTkProgressBar(
            progress_frame,
            height=10,  # Thicker progress bar
            corner_radius=5,
            progress_color="#0070f3",
            fg_color="#222222"
        )
        self.progress_bar.grid(row=1, column=0, sticky="ew", pady=(0, 10))
        self.progress_bar.set(0)
        
        # Status description
        self.status_label = ctk.CTkLabel(
            progress_frame,
            textvariable=self.status_var,
            font=ctk.CTkFont(family=self.font_family, size=12),  # Larger status description
            text_color="#888888"
        )
        self.status_label.grid(row=2, column=0, sticky="w")
        
        # Download button
        self.download_btn = ctk.CTkButton(
            content_frame,
            text="⬇️ Download & Install Extension",
            font=ctk.CTkFont(family=self.font_family, size=16, weight="bold"),  # Larger button font
            height=50,  # Taller button
            corner_radius=14,
            fg_color="#0070f3",
            hover_color="#0052cc",
            command=self.start_download
        )
        self.download_btn.grid(row=4, column=0, sticky="ew")
    
    def create_compact_logs(self, parent):
        # Compact logs section
        logs_frame = ctk.CTkFrame(
            parent,
            fg_color="#0a0a0a",
            corner_radius=16,
            border_width=1,
            border_color="#1a1a1a"
        )
        logs_frame.grid(row=2, column=0, sticky="nsew", pady=(0, 12))
        logs_frame.grid_columnconfigure(0, weight=1)
        logs_frame.grid_rowconfigure(1, weight=1)
        
        # Compact header
        header_frame = ctk.CTkFrame(
            logs_frame,
            fg_color="#111111",
            corner_radius=0,
            height=45  # Slightly taller header
        )
        header_frame.grid(row=0, column=0, sticky="ew")
        header_frame.grid_columnconfigure(1, weight=1)
        
        # Terminal dots
        dots_frame = ctk.CTkFrame(
            header_frame,
            fg_color="transparent"
        )
        dots_frame.grid(row=0, column=0, sticky="w", padx=20, pady=14)
        
        red_dot = ctk.CTkLabel(dots_frame, text="●", text_color="#ff5f56", font=ctk.CTkFont(size=12))
        red_dot.pack(side="left", padx=(0, 4))
        yellow_dot = ctk.CTkLabel(dots_frame, text="●", text_color="#ffbd2e", font=ctk.CTkFont(size=12))
        yellow_dot.pack(side="left", padx=(0, 4))
        green_dot = ctk.CTkLabel(dots_frame, text="●", text_color="#27ca3f", font=ctk.CTkFont(size=12))
        green_dot.pack(side="left")
        
        logs_title = ctk.CTkLabel(
            header_frame,
            text="🖥️ Process Terminal",
            font=ctk.CTkFont(family=self.font_family, size=14, weight="bold"),  # Larger terminal title
            text_color="#ffffff"
        )
        logs_title.grid(row=0, column=1)
        
        # Compact terminal
        self.log_textbox = ctk.CTkTextbox(
            logs_frame,
            font=ctk.CTkFont(family="Consolas" if sys.platform.startswith('win') else "Monaco", size=11),  # Larger terminal font
            fg_color="#000000",
            text_color="#00ff88",
            corner_radius=0,
            border_width=0
        )
        self.log_textbox.grid(row=1, column=0, sticky="nsew", padx=2, pady=(0, 2))
        
        # Initial compact messages
        self.log_message("╭─ Predict Assistant Extension Downloader")
        self.log_message("├─ ▶ System ready")
        self.log_message("├─ 📦 Package: xpand-technologia.zip")
        self.log_message("├─ 🌐 Source: notmastah.dev")
        self.log_message("╰─ ✨ Click download to begin")
        self.log_message("")
    
    def create_minimal_footer(self, parent):
        # Minimal footer
        footer_label = ctk.CTkLabel(
            parent,
            text="© 2025 Predict Assistant • Built with Python & CustomTkinter",
            font=ctk.CTkFont(family=self.font_family, size=11),  # Larger footer font
            text_color="#555555"
        )
        footer_label.grid(row=3, column=0, pady=(0, 12))
    
    def browse_destination(self):
        folder = filedialog.askdirectory(title="Select Installation Directory")
        if folder:
            self.destination_entry.delete(0, 'end')
            self.destination_entry.insert(0, folder)
    
    def log_message(self, message):
        timestamp = time.strftime("%H:%M:%S")
        if message.strip():  # Don't add timestamp to empty lines
            formatted_message = f"[{timestamp}] {message}\n"
        else:
            formatted_message = f"{message}\n"
        self.log_textbox.insert("end", formatted_message)
        self.log_textbox.see("end")
        self.app.update()
    
    def update_progress(self, value, step, status):
        self.progress_bar.set(value / 100.0)
        self.step_var.set(step)
        self.status_var.set(status)
        self.app.update()
    
    def start_download(self):
        if not self.destination_entry.get():
            messagebox.showerror("Error", "Please specify an installation directory")
            return
            
        if self.is_downloading:
            return
        
        self.is_downloading = True
        self.download_btn.configure(
            text="⏳ Installing...",
            state="disabled",
            fg_color="#666666"
        )
        
        # Start download in separate thread
        thread = threading.Thread(target=self.download_process)
        thread.daemon = True
        thread.start()
    
    def download_process(self):
        try:
            destination_path = self.destination_entry.get()
            
            self.log_message("")
            self.log_message("╭─ Starting installation...")
            
            # Step 1: Initialize
            self.update_progress(5, "🔧 Initializing", "Setting up environment...")
            self.log_message("├─ 🔧 Initializing environment")
            
            # Create destination directory
            os.makedirs(destination_path, exist_ok=True)
            self.log_message(f"├─ 📁 Directory: {destination_path}")
            time.sleep(0.5)
            
            # Step 2: Download
            self.update_progress(20, "⬇️ Downloading", "Connecting to servers...")
            self.log_message("├─ 🌐 Connecting to notmastah.dev...")
            
            response = requests.get(self.source_url, stream=True)
            response.raise_for_status()
            
            zip_path = "xpand_temp.zip"
            total_size = int(response.headers.get('content-length', 0))
            
            self.log_message(f"├─ 📦 Size: {total_size:,} bytes")
            
            with open(zip_path, 'wb') as f:
                downloaded = 0
                for chunk in response.iter_content(chunk_size=8192):
                    if chunk:
                        f.write(chunk)
                        downloaded += len(chunk)
                        if total_size > 0:
                            progress = 20 + (downloaded / total_size) * 40
                            percent = int((downloaded/total_size)*100)
                            self.update_progress(
                                progress, 
                                f"⬇️ Downloading ({percent}%)", 
                                f"{downloaded:,}/{total_size:,} bytes"
                            )
            
            self.log_message(f"├─ ✅ Downloaded: {os.path.getsize(zip_path):,} bytes")
            
            # Step 3: Extract
            self.update_progress(70, "📂 Extracting", "Extracting files...")
            self.log_message("├─ 📂 Extracting package...")
            
            with zipfile.ZipFile(zip_path, 'r') as zip_ref:
                file_list = zip_ref.namelist()
                file_count = len(file_list)
                
                self.log_message(f"├─ 📄 Files: {file_count}")
                
                for i, file in enumerate(file_list):
                    zip_ref.extract(file, destination_path)
                    progress = 70 + ((i + 1) / file_count) * 20
                    self.update_progress(
                        progress,
                        f"📂 Extracting ({i+1}/{file_count})",
                        f"{os.path.basename(file)}"
                    )
                    time.sleep(0.02)
            
            self.log_message(f"├─ ✅ Extracted: {file_count} files")
            
            # Step 4: Cleanup
            self.update_progress(95, "🧹 Finalizing", "Cleaning up...")
            self.log_message("├─ 🧹 Cleaning up...")
            
            os.remove(zip_path)
            time.sleep(0.3)
            
            # Complete
            self.update_progress(100, "✅ Complete", "Extension ready!")
            self.log_message("├─ ✅ Installation complete!")
            self.log_message(f"├─ 📁 Location: {os.path.abspath(destination_path)}")
            self.log_message("╰─ 🎉 Ready to use!")
            self.log_message("")
            
            # Success dialog
            messagebox.showinfo(
                "✅ Installation Complete!", 
                f"Predict Assistant extension installed successfully!\n\n📁 Location: {os.path.abspath(destination_path)}\n\n🎉 Extension is ready to use!"
            )
            
        except requests.exceptions.RequestException as e:
            self.log_message(f"├─ ❌ Download failed: {str(e)}")
            self.update_progress(0, "❌ Download Failed", "Connection error")
            messagebox.showerror("Download Error", f"Failed to download:\n\n{str(e)}")
            
        except zipfile.BadZipFile:
            self.log_message("├─ ❌ Invalid package")
            self.update_progress(0, "❌ Package Error", "Invalid package")
            messagebox.showerror("Package Error", "Invalid extension package")
            
        except Exception as e:
            self.log_message(f"├─ ❌ Error: {str(e)}")
            self.update_progress(0, "❌ Failed", "Installation error")
            messagebox.showerror("Installation Error", f"Installation failed:\n\n{str(e)}")
            
        finally:
            # Reset button
            self.is_downloading = False
            self.download_btn.configure(
                text="⬇️ Download & Install Extension",
                state="normal",
                fg_color="#0070f3"
            )

def main():
    # Check dependencies
    try:
        import customtkinter
    except ImportError:
        print("❌ CustomTkinter not installed. Run: pip install customtkinter")
        return
    
    try:
        import requests
    except ImportError:
        print("❌ Requests not installed. Run: pip install requests")
        return
    
    print("🚀 Starting Predict Assistant Extension Downloader...")
    app = CompactPredictDownloader()
    app.app.mainloop()

if __name__ == "__main__":
    main()

# PyInstaller build commands:
# pyinstaller --onefile --windowed --name "PredictAssistant Downloader" ^
# --icon="favicon.ico" ^
# --add-data "favicon.ico;." ^
# --add-data "logo-predict-p.png;." ^
# --add-data "logo-predict.png;." ^
# --add-data "Inter-Regular.otf;." ^
# --add-data "Inter-Medium.otf;." ^
# --add-data "Inter-SemiBold.otf;." ^
# --add-data "Inter-Bold.otf;." ^
# extension_downloader.py